/*! Built with http://stenciljs.com */
const { h } = window.bscomponents;

import { p as _size, q as customEvent, r as removeClass, s as addClass, t as hasClass } from './chunk-89dac778.js';
import { a as closest } from './chunk-5028a50e.js';
import { a as getTransitionDurationFromElement } from './chunk-8f426baa.js';
import './chunk-64cf3b78.js';

class BsAlert {
    constructor() {
        this.openEventName = 'open.bs.alert';
        this.openedEventName = 'opened.bs.alert';
        this.closeEventName = 'close.bs.alert';
        this.closedEventName = 'closed.bs.alert';
        this.noSelfRemoveFromDom = false;
        this.dismiss = false;
    }
    componentWillLoad() {
        if (this.dismiss === true) {
            if (!hasClass(this.alertEl, 'fade')) {
                removeClass(this.alertEl, 'show');
                if (!this.noSelfRemoveFromDom) {
                    this.alertEl.parentNode.removeChild(this.alertEl);
                }
                return;
            }
            removeClass(this.alertEl, 'fade'); // no animation when setting the initial state
            removeClass(this.alertEl, 'show');
            const transitionDuration = getTransitionDurationFromElement(this.alertEl);
            setTimeout(() => {
                addClass(this.alertEl, 'fade');
                if (!this.noSelfRemoveFromDom) {
                    this.alertEl.parentNode.removeChild(this.alertEl);
                }
            }, transitionDuration);
            this.close();
        }
    }
    handleFocusOut(event) {
        const closestDismissEl = closest(event.target, '[data-dismiss="alert"]');
        if (closestDismissEl && this.alertEl.contains(closestDismissEl)) {
            if (event.stopPropagation) {
                event.stopPropagation();
                event.preventDefault();
            }
            this.close();
        }
    }
    destroyAlert() {
        if (!this.noSelfRemoveFromDom) {
            this.alertEl.parentNode.removeChild(this.alertEl);
        }
        else {
            addClass(this.alertEl, 'd-none');
        }
        window.requestAnimationFrame(() => {
            window.requestAnimationFrame(() => {
                setTimeout(() => {
                    customEvent(this.alertEl, this.closedEventName);
                }, 0);
            });
        });
    }
    handleActiveWatch(newValue /* , oldValue */) {
        // console.log('newValue: ', newValue);
        if (newValue === true) {
            this.close();
            return;
        }
        this.open();
    }
    close() {
        if (!hasClass(this.alertEl, 'show')) {
            return;
        }
        // console.log('close');
        const closeEvent = customEvent(this.alertEl, this.closeEventName);
        if (closeEvent.defaultPrevented) {
            return;
        }
        // console.log('made it');
        // debugger;
        removeClass(this.alertEl, 'show');
        // console.log('this.alertEl: ', this.alertEl);
        if (!hasClass(this.alertEl, 'fade')) {
            this.destroyAlert();
            return;
        }
        const transitionDuration = getTransitionDurationFromElement(this.alertEl);
        setTimeout(() => {
            this.destroyAlert();
        }, transitionDuration);
    }
    open() {
        if (hasClass(this.alertEl, 'show') && !hasClass(this.alertEl, 'd-none')) {
            return;
        }
        const openEvent = customEvent(this.alertEl, this.openEventName);
        if (openEvent.defaultPrevented) {
            return;
        }
        if (this.noSelfRemoveFromDom) {
            removeClass(this.alertEl, 'd-none');
        }
        addClass(this.alertEl, 'show');
        const transitionDuration = getTransitionDurationFromElement(this.alertEl);
        setTimeout(() => {
            window.requestAnimationFrame(() => {
                window.requestAnimationFrame(() => {
                    setTimeout(() => {
                        customEvent(this.alertEl, this.openedEventName);
                    }, 0);
                });
            });
        }, transitionDuration);
    }
    alert(alertOptions) {
        if (_size(alertOptions) === 0) {
            return this.alertEl;
        }
        if (alertOptions === 'close') {
            this.close();
            return true;
        }
        if (alertOptions === 'open') {
            this.open();
            return true;
        }
        if (typeof alertOptions === 'string') {
            throw new Error(`No method named "${alertOptions}"`);
        }
        return null;
    }
    render() {
        return (h("slot", null));
    }
    static get is() { return "bs-alert"; }
    static get properties() { return {
        "alert": {
            "method": true
        },
        "alertEl": {
            "elementRef": true
        },
        "close": {
            "method": true
        },
        "closedEventName": {
            "type": String,
            "attr": "closed-event-name"
        },
        "closeEventName": {
            "type": String,
            "attr": "close-event-name"
        },
        "dismiss": {
            "type": Boolean,
            "attr": "dismiss",
            "mutable": true,
            "watchCallbacks": ["handleActiveWatch"]
        },
        "noSelfRemoveFromDom": {
            "type": Boolean,
            "attr": "no-self-remove-from-dom"
        },
        "open": {
            "method": true
        },
        "openedEventName": {
            "type": String,
            "attr": "opened-event-name"
        },
        "openEventName": {
            "type": String,
            "attr": "open-event-name"
        }
    }; }
    static get listeners() { return [{
            "name": "click",
            "method": "handleFocusOut"
        }]; }
    static get style() { return "bs-alert {\n  display: block;\n}"; }
}

export { BsAlert };
