/*! Built with http://stenciljs.com */
import { h } from './bscomponents.core.js';
import { p as _size, q as customEvent, r as removeClass, s as addClass, t as hasClass } from './chunk-89dac778.js';
import { a as closest } from './chunk-5028a50e.js';
import { a as getTransitionDurationFromElement } from './chunk-8f426baa.js';
import './chunk-64cf3b78.js';
var BsAlert = /** @class */ (function () {
    function BsAlert() {
        this.openEventName = 'open.bs.alert';
        this.openedEventName = 'opened.bs.alert';
        this.closeEventName = 'close.bs.alert';
        this.closedEventName = 'closed.bs.alert';
        this.noSelfRemoveFromDom = false;
        this.dismiss = false;
    }
    BsAlert.prototype.componentWillLoad = function () {
        var _this = this;
        if (this.dismiss === true) {
            if (!hasClass(this.alertEl, 'fade')) {
                removeClass(this.alertEl, 'show');
                if (!this.noSelfRemoveFromDom) {
                    this.alertEl.parentNode.removeChild(this.alertEl);
                }
                return;
            }
            removeClass(this.alertEl, 'fade'); // no animation when setting the initial state
            removeClass(this.alertEl, 'show');
            var transitionDuration = getTransitionDurationFromElement(this.alertEl);
            setTimeout(function () {
                addClass(_this.alertEl, 'fade');
                if (!_this.noSelfRemoveFromDom) {
                    _this.alertEl.parentNode.removeChild(_this.alertEl);
                }
            }, transitionDuration);
            this.close();
        }
    };
    BsAlert.prototype.handleFocusOut = function (event) {
        var closestDismissEl = closest(event.target, '[data-dismiss="alert"]');
        if (closestDismissEl && this.alertEl.contains(closestDismissEl)) {
            if (event.stopPropagation) {
                event.stopPropagation();
                event.preventDefault();
            }
            this.close();
        }
    };
    BsAlert.prototype.destroyAlert = function () {
        var _this = this;
        if (!this.noSelfRemoveFromDom) {
            this.alertEl.parentNode.removeChild(this.alertEl);
        }
        else {
            addClass(this.alertEl, 'd-none');
        }
        window.requestAnimationFrame(function () {
            window.requestAnimationFrame(function () {
                setTimeout(function () {
                    customEvent(_this.alertEl, _this.closedEventName);
                }, 0);
            });
        });
    };
    BsAlert.prototype.handleActiveWatch = function (newValue /* , oldValue */) {
        // console.log('newValue: ', newValue);
        if (newValue === true) {
            this.close();
            return;
        }
        this.open();
    };
    BsAlert.prototype.close = function () {
        var _this = this;
        if (!hasClass(this.alertEl, 'show')) {
            return;
        }
        // console.log('close');
        var closeEvent = customEvent(this.alertEl, this.closeEventName);
        if (closeEvent.defaultPrevented) {
            return;
        }
        // console.log('made it');
        // debugger;
        removeClass(this.alertEl, 'show');
        // console.log('this.alertEl: ', this.alertEl);
        if (!hasClass(this.alertEl, 'fade')) {
            this.destroyAlert();
            return;
        }
        var transitionDuration = getTransitionDurationFromElement(this.alertEl);
        setTimeout(function () {
            _this.destroyAlert();
        }, transitionDuration);
    };
    BsAlert.prototype.open = function () {
        var _this = this;
        if (hasClass(this.alertEl, 'show') && !hasClass(this.alertEl, 'd-none')) {
            return;
        }
        var openEvent = customEvent(this.alertEl, this.openEventName);
        if (openEvent.defaultPrevented) {
            return;
        }
        if (this.noSelfRemoveFromDom) {
            removeClass(this.alertEl, 'd-none');
        }
        addClass(this.alertEl, 'show');
        var transitionDuration = getTransitionDurationFromElement(this.alertEl);
        setTimeout(function () {
            window.requestAnimationFrame(function () {
                window.requestAnimationFrame(function () {
                    setTimeout(function () {
                        customEvent(_this.alertEl, _this.openedEventName);
                    }, 0);
                });
            });
        }, transitionDuration);
    };
    BsAlert.prototype.alert = function (alertOptions) {
        if (_size(alertOptions) === 0) {
            return this.alertEl;
        }
        if (alertOptions === 'close') {
            this.close();
            return true;
        }
        if (alertOptions === 'open') {
            this.open();
            return true;
        }
        if (typeof alertOptions === 'string') {
            throw new Error("No method named \"" + alertOptions + "\"");
        }
        return null;
    };
    BsAlert.prototype.render = function () {
        return (h("slot", null));
    };
    Object.defineProperty(BsAlert, "is", {
        get: function () { return "bs-alert"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsAlert, "properties", {
        get: function () {
            return {
                "alert": {
                    "method": true
                },
                "alertEl": {
                    "elementRef": true
                },
                "close": {
                    "method": true
                },
                "closedEventName": {
                    "type": String,
                    "attr": "closed-event-name"
                },
                "closeEventName": {
                    "type": String,
                    "attr": "close-event-name"
                },
                "dismiss": {
                    "type": Boolean,
                    "attr": "dismiss",
                    "mutable": true,
                    "watchCallbacks": ["handleActiveWatch"]
                },
                "noSelfRemoveFromDom": {
                    "type": Boolean,
                    "attr": "no-self-remove-from-dom"
                },
                "open": {
                    "method": true
                },
                "openedEventName": {
                    "type": String,
                    "attr": "opened-event-name"
                },
                "openEventName": {
                    "type": String,
                    "attr": "open-event-name"
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsAlert, "listeners", {
        get: function () {
            return [{
                    "name": "click",
                    "method": "handleFocusOut"
                }];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsAlert, "style", {
        get: function () { return "bs-alert {\n  display: block;\n}"; },
        enumerable: true,
        configurable: true
    });
    return BsAlert;
}());
export { BsAlert };
