/*! Built with http://stenciljs.com */
import { h } from './bscomponents.core.js';
import { a as closest } from './chunk-5028a50e.js';
import { a as Popper } from './chunk-f2a0d5d9.js';
import { p as _size, o as toLower, s as addClass, r as removeClass, t as hasClass, q as customEvent } from './chunk-89dac778.js';
import { c as _has } from './chunk-93432030.js';
import { a as getUniqueId } from './chunk-ddbf79d9.js';
import { a as getTransitionDurationFromElement } from './chunk-8f426baa.js';
import { a as getConfigBoolean } from './chunk-c1b19ebe.js';
import './chunk-64cf3b78.js';
function clickWasInside(el, selector) {
    return (closest(el, selector) !== null);
}
var BsDropdown = /** @class */ (function () {
    function BsDropdown() {
        var _this = this;
        this.showEventName = 'show.bs.dropdown';
        this.shownEventName = 'shown.bs.dropdown';
        this.hideEventName = 'hide.bs.dropdown';
        this.hiddenEventName = 'hidden.bs.dropdown';
        this.focusoutEventName = 'focusout.bs.dropdown';
        this.showDropdown = false;
        this.show = false;
        this.config = {};
        this.defaults = {
            offset: 0,
            flip: true,
            boundary: 'scrollParent',
            reference: 'toggle',
            display: 'dynamic',
        };
        this.handleDropdownClickOutside = function (event) {
            // console.log('event: ', event);
            // https://gist.github.com/Restuta/e400a555ba24daa396cc
            // handleDropdownClickOutside is an arrow function so that removeEventListener will work
            if (!_this.show) {
                return;
            }
            var clickWasInsideDropdown = clickWasInside(event.target, "[data-bs-id=\"" + _this.dropdownId + "\"]");
            if (clickWasInsideDropdown) {
                var closestForm = closest(event.target, 'form');
                if (_this.dropdownEl.contains(closestForm)) {
                    // the user clicked on a form that is inside the dropdown
                    // so keep the dropdown open
                    return;
                }
                if (toLower(event.target.tagName) === 'input' || toLower(event.target.tagName) === 'textarea') {
                    return;
                }
            }
            document.removeEventListener('click', _this.handleDropdownClickOutside);
            _this.show = true;
            _this.handleHideDropdown();
        };
        this.handleToggleBsDropdown = function (event) {
            event.preventDefault();
            if (_this.show === true) {
                _this.handleHideDropdown();
            }
            else {
                _this.handleShowDropdown(event.target);
            }
        };
    }
    BsDropdown.prototype.componentWillLoad = function () {
        var _this = this;
        if (hasClass(this.dropdownEl, 'show') && this.show === false) {
            this.show = true;
        }
        if (this.show === true) {
            setTimeout(function () {
                document.removeEventListener('click', _this.handleDropdownClickOutside);
                document.addEventListener('click', _this.handleDropdownClickOutside);
            }, 0);
        }
        this.dropdownId = getUniqueId('dropdown');
        this.dropdownEl.dataset.bsId = this.dropdownId;
        var toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        for (var j = 0, len = toggles.length; j < len; j += 1) {
            toggles[j].removeEventListener('click', this.handleToggleBsDropdown);
            toggles[j].addEventListener('click', this.handleToggleBsDropdown);
        }
        // obey initial show-dropdown prop
        if (this.showDropdown === true && this.show === false) {
            var dropdownMenuEl_1 = this.dropdownEl.querySelector('.dropdown-menu');
            if (!dropdownMenuEl_1) {
                return;
            }
            var dropdownMenuTransitionDuration = getTransitionDurationFromElement(dropdownMenuEl_1);
            if (hasClass(dropdownMenuEl_1, 'fade')) {
                removeClass(dropdownMenuEl_1, 'fade');
                this.handleShowDropdown();
                setTimeout(function () {
                    addClass(dropdownMenuEl_1, 'fade');
                }, dropdownMenuTransitionDuration);
            }
            else {
                this.handleShowDropdown();
            }
        }
        else if (this.showDropdown === false && this.show === true) {
            this.handleHideDropdown();
        }
    };
    BsDropdown.prototype.componentDidUnload = function () {
        this.dispose();
    };
    BsDropdown.prototype.dispose = function () {
        if (this.show === true) {
            this.handleHideDropdown();
        }
        document.removeEventListener('click', this.handleDropdownClickOutside);
        var toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        this.relatedTarget = null;
        for (var j = 0, len = toggles.length; j < len; j += 1) {
            toggles[j].removeEventListener('click', this.handleToggleBsDropdown);
        }
        if (this.popperHandle && this.popperHandle.destroy) {
            this.popperHandle.destroy();
            this.popperHandle = null;
        }
    };
    BsDropdown.prototype.initPopper = function (dropdownMenuEl) {
        this.inNavbar = BsDropdown.detectNavbar(this.dropdownEl);
        if (!this.inNavbar) {
            var popperConfig = BsDropdown.getPopperDropdownConfig(this.dropdownEl, dropdownMenuEl, this.config);
            // If boundary is not `scrollParent`, then set position to `static`
            // to allow the menu to "escape" the scroll parent's boundaries
            // https://github.com/twbs/bootstrap/issues/24251
            if (this.config.boundary !== 'scrollParent') {
                addClass(this.dropdownEl, 'position-static');
            }
            var referenceElement = this.relatedTarget;
            if (this.config.reference === 'parent') {
                referenceElement = this.relatedTarget.parentNode;
            }
            else if (this.config.reference instanceof Element) {
                referenceElement = this.config.reference;
            }
            // console.log('this.config: ', this.config);
            // console.log('referenceElement: ', referenceElement);
            // console.log('dropdownMenuEl: ', dropdownMenuEl);
            // console.log('popperConfig: ', popperConfig);
            this.popperHandle = new Popper(referenceElement, dropdownMenuEl, popperConfig);
        }
    };
    BsDropdown.detectNavbar = function (el) {
        return closest(el, '.navbar') !== null;
    };
    BsDropdown.prototype.setConfig = function (relatedTarget) {
        this.config = {};
        var config = {};
        if (_has(relatedTarget.dataset, 'offset')) {
            config.offset = relatedTarget.dataset.offset;
        }
        else {
            config.offset = this.defaults.offset;
        }
        if (_has(relatedTarget.dataset, 'relatedTarget')) {
            config.flip = getConfigBoolean(relatedTarget.dataset.flip);
        }
        else {
            config.flip = this.defaults.flip;
        }
        if (_has(relatedTarget.dataset, 'boundary')) {
            config.boundary = relatedTarget.dataset.boundary;
        }
        else {
            config.boundary = this.defaults.boundary;
        }
        if (_has(relatedTarget.dataset, 'reference')) {
            config.reference = relatedTarget.dataset.reference;
        }
        else {
            config.reference = this.defaults.reference;
        }
        if (_has(relatedTarget.dataset, 'display')) {
            config.display = relatedTarget.dataset.display;
        }
        else {
            config.display = this.defaults.display;
        }
        this.config = config;
        // console.log('this.config: ', this.config);
    };
    BsDropdown.prototype.getAssumedRelatedTarget = function () {
        var assumedRelatedTarget = this.dropdownEl.querySelector('[data-toggle="dropdown"]');
        if (assumedRelatedTarget) {
            return assumedRelatedTarget;
        }
        return this.dropdownEl;
    };
    BsDropdown.prototype.handleShowDropdown = function (passedRelatedTarget) {
        var _this = this;
        if (passedRelatedTarget === void 0) { passedRelatedTarget = null; }
        // console.log('handleShowDropdown relatedTarget: ', relatedTarget);
        this.relatedTarget = passedRelatedTarget || this.getAssumedRelatedTarget();
        if (this.relatedTarget.hasAttribute('disabled') || hasClass(this.relatedTarget, 'disabled')) {
            return;
        }
        var showEvent = customEvent(this.dropdownEl, this.showEventName, {}, this.relatedTarget);
        this.setConfig(this.relatedTarget);
        if (this.show || showEvent.defaultPrevented) {
            return;
        }
        this.show = true;
        var dropdownMenuEl = this.dropdownEl.querySelector('.dropdown-menu');
        if (!dropdownMenuEl) {
            return;
        }
        // const toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        addClass(this.dropdownEl, 'show');
        addClass(dropdownMenuEl, 'show');
        this.relatedTarget.setAttribute('aria-expanded', 'true');
        // for (let j = 0, len = toggles.length; j < len; j += 1) {
        //   toggles[j].setAttribute('aria-expanded', 'true');
        // }
        document.removeEventListener('click', this.handleDropdownClickOutside);
        setTimeout(function () {
            document.removeEventListener('click', _this.handleDropdownClickOutside);
            document.addEventListener('click', _this.handleDropdownClickOutside);
        }, 0);
        var dropdownMenuTransitionDuration = getTransitionDurationFromElement(dropdownMenuEl);
        this.initPopper(dropdownMenuEl);
        setTimeout(function () {
            window.requestAnimationFrame(function () {
                window.requestAnimationFrame(function () {
                    customEvent(_this.dropdownEl, _this.shownEventName, {}, _this.relatedTarget);
                });
            });
        }, dropdownMenuTransitionDuration);
    };
    BsDropdown.prototype.handleHideDropdown = function () {
        var _this = this;
        // console.log('handleHideDropdown relatedTarget: ', relatedTarget);
        var hideEvent = customEvent(this.dropdownEl, this.hideEventName, {}, this.relatedTarget);
        if (!this.show || hideEvent.defaultPrevented) {
            return;
        }
        this.show = false;
        var dropdownMenuEl = this.dropdownEl.querySelector('.dropdown-menu');
        if (!dropdownMenuEl) {
            return;
        }
        var toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        document.removeEventListener('click', this.handleDropdownClickOutside);
        removeClass(this.dropdownEl, 'show');
        removeClass(dropdownMenuEl, 'show');
        for (var j = 0, len = toggles.length; j < len; j += 1) {
            toggles[j].setAttribute('aria-expanded', 'false');
        }
        var dropdownMenuTransitionDuration = getTransitionDurationFromElement(dropdownMenuEl);
        setTimeout(function () {
            window.requestAnimationFrame(function () {
                window.requestAnimationFrame(function () {
                    customEvent(_this.dropdownEl, _this.hiddenEventName, {}, _this.relatedTarget);
                });
            });
            _this.relatedTarget = null;
        }, dropdownMenuTransitionDuration);
    };
    BsDropdown.getPlacement = function (dropdownEl, dropdownMenuEl) {
        var AttachmentMap = {
            top: 'top-start',
            topend: 'top-end',
            bottom: 'bottom-start',
            bottomend: 'bottom-end',
            right: 'right-start',
            // RIGHTEND: 'right-end',
            left: 'left-start',
        };
        var ClassName = {
            // DISABLED: 'disabled',
            // SHOW: 'show',
            dropup: 'dropup',
            dropright: 'dropright',
            dropleft: 'dropleft',
            menuright: 'dropdown-menu-right',
        };
        var placement = AttachmentMap.bottom;
        if (hasClass(dropdownEl, ClassName.dropup)) {
            placement = AttachmentMap.top;
            if (hasClass(dropdownMenuEl, ClassName.menuright)) {
                placement = AttachmentMap.topend;
            }
        }
        else if (hasClass(dropdownEl, ClassName.dropright)) {
            placement = AttachmentMap.right;
        }
        else if (hasClass(dropdownEl, ClassName.dropleft)) {
            placement = AttachmentMap.left;
        }
        else if (hasClass(dropdownEl, ClassName.menuright)) {
            placement = AttachmentMap.bottomend;
        }
        // console.log('placement: ', placement);
        return placement;
    };
    BsDropdown.getPopperDropdownConfig = function (dropdownEl, dropdownMenuEl, popperSettings) {
        var popperConfig = {
            placement: BsDropdown.getPlacement(dropdownEl, dropdownMenuEl),
            modifiers: {
                offset: { offset: popperSettings.offset },
                flip: {
                    enabled: popperSettings.flip,
                },
                preventOverflow: {
                    boundariesElement: popperSettings.boundary,
                },
            },
        };
        // Disable Popper.js if we have a static display
        if (popperSettings.display === 'static') {
            popperConfig.modifiers.applyStyle = {
                enabled: false,
            };
        }
        return popperConfig;
    };
    BsDropdown.prototype.handleFocusOut = function (event) {
        var _this = this;
        // console.log('event: ', event);
        if (!event.relatedTarget) {
            return;
        }
        var customFocusoutEvent = customEvent(this.dropdownEl, this.focusoutEventName, { originalFocusoutEventTarget: event.target }, event.relatedTarget);
        if (customFocusoutEvent.defaultPrevented) {
            return;
        }
        if (this.show && !this.dropdownEl.contains(event.relatedTarget)) {
            window.requestAnimationFrame(function () {
                window.requestAnimationFrame(function () {
                    setTimeout(function () {
                        _this.handleHideDropdown();
                    }, 0);
                });
            });
        }
    };
    BsDropdown.prototype.keydownIsADropdownCommand = function (event) {
        var ESCAPE_KEYCODE = 27; // KeyboardEvent.which value for Escape (Esc) key
        var SPACE_KEYCODE = 32; // KeyboardEvent.which value for space key
        var ARROW_UP_KEYCODE = 38; // KeyboardEvent.which value for up arrow key
        var ARROW_DOWN_KEYCODE = 40; // KeyboardEvent.which value for down arrow key
        if (toLower(event.target.tagName) === 'input' || toLower(event.target.tagName) === 'textarea') {
            if (event.which === SPACE_KEYCODE) {
                return false;
            }
            if (event.which === ESCAPE_KEYCODE) {
                return true;
            }
            var dropdownMenu = this.dropdownEl.querySelector('.dropdown-menu');
            if (dropdownMenu.contains(event.target)) {
                return false;
            }
            if (event.which === ARROW_DOWN_KEYCODE) {
                if (event.target.dataset.toggle === 'dropdown') {
                    return true;
                }
                return false;
            }
            if (event.which === ARROW_UP_KEYCODE) {
                if (event.target.dataset.toggle === 'dropdown') {
                    return true;
                }
                return false;
            }
        }
        else {
            if (event.which === ARROW_UP_KEYCODE) {
                return true;
            }
            if (event.which === ARROW_DOWN_KEYCODE) {
                return true;
            }
            if (event.which === ESCAPE_KEYCODE) {
                return true;
            }
            if (event.which === SPACE_KEYCODE) {
                return true;
            }
        }
        return false;
    };
    BsDropdown.prototype.handleKeyDown = function (event) {
        // console.log('handleKeyDown event: ', event);
        var ESCAPE_KEYCODE = 27; // KeyboardEvent.which value for Escape (Esc) key
        var SPACE_KEYCODE = 32; // KeyboardEvent.which value for space key
        // const TAB_KEYCODE = 9; // KeyboardEvent.which value for tab key
        var ARROW_UP_KEYCODE = 38; // KeyboardEvent.which value for up arrow key
        var ARROW_DOWN_KEYCODE = 40; // KeyboardEvent.which value for down arrow key
        // const RIGHT_MOUSE_BUTTON_WHICH = 3; // MouseEvent.which value for the right button (assuming a right-handed mouse)
        // const REGEXP_KEYDOWN = new RegExp(ARROW_UP_KEYCODE + "|" + ARROW_DOWN_KEYCODE + "|" + ESCAPE_KEYCODE);
        if (!this.keydownIsADropdownCommand(event)) {
            return;
        }
        event.preventDefault();
        event.stopPropagation();
        if (event.target.hasAttribute('disabled') || hasClass(event.target, 'disabled')) {
            return;
        }
        var relatedTarget = this.getAssumedRelatedTarget();
        if (event.which === ESCAPE_KEYCODE) {
            if (this.show === true) {
                this.handleHideDropdown();
            }
            else {
                this.handleShowDropdown(relatedTarget);
            }
            relatedTarget.focus();
            return;
        }
        if (event.which === SPACE_KEYCODE) {
            if (relatedTarget === document.activeElement) {
                if (this.show === true) {
                    this.handleHideDropdown();
                }
                else {
                    this.handleShowDropdown(relatedTarget);
                }
            }
            return;
        }
        if (event.which === ARROW_DOWN_KEYCODE && !this.show) {
            this.handleShowDropdown(relatedTarget);
            relatedTarget.focus();
            return;
        }
        if (event.which === ARROW_UP_KEYCODE && !this.show) {
            this.handleShowDropdown(relatedTarget);
            relatedTarget.focus();
            return;
        }
        if (event.which !== 38 && event.which !== 40) {
            console.warn('unhandled event.which: ', event.which);
        }
        var dropdownMenuItems = Array.prototype.slice.call(this.dropdownEl.querySelectorAll('.dropdown-menu .dropdown-item:not(.disabled):not(:disabled)'));
        if (dropdownMenuItems.length === 0) {
            return;
        }
        var myIndex = dropdownMenuItems.indexOf(document.activeElement);
        if (event.which === ARROW_UP_KEYCODE && myIndex > 0) { // Up
            myIndex -= 1;
        }
        if (event.which === ARROW_DOWN_KEYCODE && myIndex < dropdownMenuItems.length - 1) { // Down
            myIndex += 1;
        }
        if (myIndex < 0) {
            myIndex = 0;
        }
        dropdownMenuItems[myIndex].focus();
    };
    BsDropdown.prototype.handlePresentWatch = function (newValue /* , oldValue */) {
        // console.log('newValue: ', newValue);
        // console.log('this.show: ', this.show);
        if (newValue && this.show) {
            return;
        }
        if (!newValue && !this.show) {
            return;
        }
        if (newValue && !this.show) {
            this.handleShowDropdown();
            return;
        }
        if (!newValue && this.show) {
            this.handleHideDropdown();
        }
    };
    BsDropdown.prototype.dropdown = function (dropdownOptions, relatedTarget) {
        if (dropdownOptions === void 0) { dropdownOptions = {}; }
        if (relatedTarget === void 0) { relatedTarget = null; }
        if (_size(dropdownOptions) === 0) {
            return this.dropdownEl;
        }
        if (dropdownOptions === 'toggle') {
            if (this.show === true) {
                this.handleHideDropdown();
            }
            else {
                this.handleShowDropdown(relatedTarget);
            }
            return true;
        }
        if (dropdownOptions === 'update') {
            this.inNavbar = BsDropdown.detectNavbar(this.dropdownEl);
            if (this.popperHandle !== null) {
                this.popperHandle.scheduleUpdate();
            }
            return true;
        }
        // if (dropdownOptions === 'dispose') {
        //   this.dispose();
        //   return true;
        // }
        if (typeof dropdownOptions === 'string') {
            throw new Error("No method named \"" + dropdownOptions + "\"");
        }
        return null;
    };
    BsDropdown.prototype.render = function () {
        return (h("slot", null));
    };
    Object.defineProperty(BsDropdown, "is", {
        get: function () { return "bs-dropdown"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsDropdown, "properties", {
        get: function () {
            return {
                "config": {
                    "type": "Any",
                    "attr": "config",
                    "mutable": true
                },
                "defaults": {
                    "type": "Any",
                    "attr": "defaults"
                },
                "dropdown": {
                    "method": true
                },
                "dropdownEl": {
                    "elementRef": true
                },
                "dropdownId": {
                    "state": true
                },
                "focusoutEventName": {
                    "type": String,
                    "attr": "focusout-event-name"
                },
                "hiddenEventName": {
                    "type": String,
                    "attr": "hidden-event-name"
                },
                "hideEventName": {
                    "type": String,
                    "attr": "hide-event-name"
                },
                "inNavbar": {
                    "state": true
                },
                "popperHandle": {
                    "state": true
                },
                "relatedTarget": {
                    "state": true
                },
                "show": {
                    "type": Boolean,
                    "attr": "show",
                    "mutable": true
                },
                "showDropdown": {
                    "type": Boolean,
                    "attr": "show-dropdown",
                    "mutable": true,
                    "watchCallbacks": ["handlePresentWatch"]
                },
                "showEventName": {
                    "type": String,
                    "attr": "show-event-name"
                },
                "shownEventName": {
                    "type": String,
                    "attr": "shown-event-name"
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsDropdown, "listeners", {
        get: function () {
            return [{
                    "name": "focusout",
                    "method": "handleFocusOut"
                }, {
                    "name": "keydown",
                    "method": "handleKeyDown"
                }];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsDropdown, "style", {
        get: function () { return "bs-dropdown {\n  display: block;\n}"; },
        enumerable: true,
        configurable: true
    });
    return BsDropdown;
}());
export { BsDropdown };
