/*! Built with http://stenciljs.com */
import { h } from './bscomponents.core.js';
import { p as _size, s as addClass, t as hasClass, r as removeClass, q as customEvent } from './chunk-89dac778.js';
import { c as _has } from './chunk-93432030.js';
import { c as toNumber, d as isNaN$1 } from './chunk-64cf3b78.js';
import { a as _isElement } from './chunk-55988a92.js';
import { a as getUniqueId } from './chunk-ddbf79d9.js';
import { a as closest, b as elementMatches } from './chunk-5028a50e.js';
import { a as getTargetSelector } from './chunk-e1923c80.js';
import './chunk-5260d582.js';
var BsScrollspy = /** @class */ (function () {
    function BsScrollspy() {
        var _this = this;
        this.activateEventName = 'activate.bs.scrollspy';
        this.useBodyForScrollElement = false;
        this.verboseLogging = false;
        this.defaults = {
            offset: 10,
            method: 'auto',
            target: '',
        };
        this.handleScrollElement = function () { return _this.process(); };
    }
    BsScrollspy.prototype.componentWillLoad = function () {
        this.scrollElement = this.useBodyForScrollElement === true ? window : this.scrollspyEl;
        this.config = this.getConfig();
        this.selector = this.config.target + " .nav-link," + this.config.target + " .list-group-item," + this.config.target + " .dropdown-item";
        this.offsets = [];
        this.targets = [];
        this.activeTarget = null;
        this.scrollHeight = 0;
        this.scrollElement.addEventListener('scroll', this.handleScrollElement);
        this.refresh();
        this.process();
    };
    BsScrollspy.prototype.componentDidUnload = function () {
        if (this.scrollElement) {
            this.scrollElement.removeEventListener('scroll', this.handleScrollElement);
        }
        this.scrollElement = null;
        this.config = null;
        this.selector = null;
        this.offsets = null;
        this.targets = null;
        this.activeTarget = null;
        this.scrollHeight = null;
    };
    BsScrollspy.prototype.getConfig = function (overrideConfig) {
        if (overrideConfig === void 0) { overrideConfig = {}; }
        var config = {};
        if (_has(overrideConfig, 'target')) {
            config.target = overrideConfig.target;
        }
        else if (_has(this.scrollspyEl.dataset, 'target')) {
            config.target = this.scrollspyEl.dataset.target;
        }
        else {
            config.target = this.defaults.target;
        }
        if (_isElement(config.target)) {
            var id = config.target.getAttribute('id');
            if (!id) {
                id = getUniqueId('scrollspy');
                config.target.setAttribute('id', id);
            }
            config.target = "#" + id;
        }
        if (_has(overrideConfig, 'offset')) {
            config.offset = toNumber(overrideConfig.offset);
        }
        else if (_has(this.scrollspyEl.dataset, 'offset')) {
            config.offset = toNumber(this.scrollspyEl.dataset.offset);
        }
        else {
            config.offset = this.defaults.offset;
        }
        if (isNaN$1(config.offset)) {
            config.offset = this.defaults.offset;
        }
        if (_has(overrideConfig, 'method')) {
            config.method = toNumber(overrideConfig.method);
        }
        else if (_has(this.scrollspyEl.dataset, 'method')) {
            config.method = this.scrollspyEl.dataset.method;
        }
        else {
            config.method = this.defaults.method;
        }
        return config;
    };
    BsScrollspy.getCalculatedStyle = function (elem, key) {
        return window.getComputedStyle(elem)[key];
    };
    // static calculatedPosition(elem) {
    //   const offset = elem.offset();
    //   // Account for the *real* offset parent, which can be the document or its root element
    //   // when a statically positioned element is identified
    //   const doc = elem.ownerDocument;
    //   let offsetParent = elem.offsetParent || doc.documentElement;
    //   while (offsetParent && (offsetParent === doc.body || offsetParent === doc.documentElement) && BsScrollspy.getCalculatedStyle(offsetParent, 'position') === 'static') {
    //     offsetParent = offsetParent.parentNode;
    //   }
    //   if (offsetParent && offsetParent !== elem && offsetParent.nodeType === 1) {
    //     // Incorporate borders into its offset, since they are outside its content origin
    //     const parentOffset = offsetParent.offset(); // https://github.com/jquery/jquery/blob/e743cbd28553267f955f71ea7248377915613fd9/src/offset.js#L74
    //     parentOffset.top += BsScrollspy.getCalculatedStyle(offsetParent, 'borderTopWidth');
    //     parentOffset.left += BsScrollspy.getCalculatedStyle(offsetParent, 'borderLeftWidth');
    //   }
    // }
    // static findPos(obj) {
    //   let curleft = 0;
    //   let curtop = 0;
    //   if (obj.offsetParent) {
    //     do {
    //       curleft += obj.offsetLeft;
    //       curtop += obj.offsetTop;
    //     } while (obj = obj.offsetParent);
    //   }
    //   return [curleft, curtop];
    // }
    BsScrollspy.offset = function (el) {
        var rect = el.getBoundingClientRect();
        var scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;
        var scrollTop = window.pageYOffset || document.documentElement.scrollTop;
        return { top: rect.top + scrollTop, left: rect.left + scrollLeft };
    };
    BsScrollspy.prototype.refresh = function () {
        var _this = this;
        if (!this.config.target || _size(this.config.target) === 0) {
            return;
        }
        var autoMethod = this.scrollElement === this.scrollElement.window ? 'offset' : 'position';
        var offsetMethod = this.config.method === 'auto' ? autoMethod : this.config.method;
        var offsetBase = offsetMethod === 'position' ? this.getScrollTop() : 0;
        // console.log('offsetBase: ', offsetBase);
        // console.log('offsetMethod: ', offsetMethod);
        // console.log('this.scrollElement: ', this.scrollElement);
        // console.log('this.scrollElement.window: ', this.scrollElement.window);
        this.offsets = [];
        this.targets = [];
        this.scrollHeight = this.getScrollHeight();
        // console.log('this.scrollHeight: ', this.scrollHeight);
        var scrollspyBCR = this.scrollElement.getBoundingClientRect();
        var container = document.querySelector(this.config.target);
        var targets = Array.prototype.slice.call(container.querySelectorAll(this.selector));
        // const scrollspyCalculatedPosition = BsScrollspy.getCalculatedStyle(this.scrollspyEl, 'position');
        targets.map(function (element) {
            var target;
            var targetSelector = getTargetSelector(element);
            if (targetSelector) {
                target = document.querySelector(targetSelector);
            }
            if (target) {
                // console.log('target: ', target);
                var targetBCR = target.getBoundingClientRect();
                if (targetBCR.width || targetBCR.height) {
                    // const returnVal = [targetBCR.top - scrollspyBCR.top + offsetBase, targetSelector];
                    // https://github.com/jquery/jquery/blob/e743cbd28553267f955f71ea7248377915613fd9/src/offset.js#L20
                    // https://github.com/jquery/jquery/blob/e743cbd28553267f955f71ea7248377915613fd9/src/offset.js#L110
                    // const returnVal = [targetBCR.top + offsetBase, targetSelector];
                    // const targetCalculatedPosition = BsScrollspy.getCalculatedStyle(target, 'position');
                    // console.log('targetCalculatedPosition: ', targetCalculatedPosition);
                    // console.log('targetBCR.top: ', targetBCR.top);
                    // console.log('scrollspyBCR.top: ', scrollspyBCR.top);
                    // console.log('target.offsetTop: ', target.offsetTop);
                    // console.log('this.scrollspyEl.offsetTop: ', this.scrollspyEl.offsetTop);
                    // let myOffsetTop;
                    // if (scrollTop > 0) {
                    //   myOffsetTop = targetBCR.top - scrollTop;
                    // } else {
                    // }
                    var myOffsetTop = targetBCR.top - scrollspyBCR.top;
                    // console.log('*** myOffsetTop: ', myOffsetTop);
                    // console.log('scrollspyCalculatedPosition: ', scrollspyCalculatedPosition);
                    // console.log('this.getScrollTop(): ', this.getScrollTop());
                    // if (scrollspyCalculatedPosition === 'relative') {
                    //   // console.log('targetBCR.top: ', targetBCR.top);
                    //   // console.log('target.offsetTop: ', target.offsetTop);
                    //   // console.log('scrollspyBCR.top: ', scrollspyBCR.top);
                    //   // console.log('this.getScrollTop(): ', this.getScrollTop());
                    //   // myOffsetTop = target.offsetTop - this.scrollspyEl.offsetTop;
                    //   // myOffsetTop = targetBCR.top - scrollspyBCR.top;
                    //   myOffsetTop = targetBCR.top - this.getScrollTop();
                    //   // myOffsetTop = mezr.offset(target, this.scrollspyEl).top;
                    //   // console.log('mezr myOffsetTop: ', myOffsetTop);
                    // } else if (scrollspyCalculatedPosition === 'static') {
                    //   // myOffsetTop = targetBCR.top;
                    //   myOffsetTop = targetBCR.top - this.getScrollTop();
                    //   // myOffsetTop = targetBCR.top - scrollspyBCR.top;
                    // } else {
                    //   // https://github.com/jquery/jquery/blob/e743cbd28553267f955f71ea7248377915613fd9/src/offset.js#L122
                    //   myOffsetTop = targetBCR.top;
                    // }
                    // const returnVal = [myOffsetTop + offsetBase, targetSelector];
                    var returnVal = [myOffsetTop, targetSelector];
                    // console.log('target: ', target);
                    // console.log("BsScrollspy.getCalculatedStyle(target, 'position'): ", BsScrollspy.getCalculatedStyle(target, 'position'));
                    // console.log("BsScrollspy.getCalculatedStyle(this.scrollspyEl, 'position'): ", BsScrollspy.getCalculatedStyle(this.scrollspyEl, 'position'));
                    // console.log('target.offsetParent: ', target.offsetParent);
                    // // console.log('target.getOffsetParent(): ', target.getOffsetParent());
                    // console.log('BsScrollspy.offset(target): ', BsScrollspy.offset(target));
                    // console.log('targetBCR: ', targetBCR);
                    // // console.log("mezr.offset(target, 'content'): ", mezr.offset(target, 'content'));
                    // // console.log("mezr.offset(target, 'padding'): ", mezr.offset(target, 'padding'));
                    // // console.log("mezr.offset(target, 'scroll'): ", mezr.offset(target, 'scroll'));
                    // // console.log("mezr.offset(target, 'border'): ", mezr.offset(target, 'border'));
                    // // console.log('mezr.offset(target, this.scrollspyEl): ', mezr.offset(target, this.scrollspyEl));
                    // console.log('mezr.offset(target, this.scrollspyEl): ', mezr.offset(target, this.scrollspyEl));
                    // // console.log('mezr.offset(target, mezr.containingBlock(this.scrollspyEl)): ', mezr.offset(target, mezr.containingBlock(this.scrollspyEl)));
                    // console.log('mezr.offset(target, mezr.containingBlock(this.scrollspyEl, "absolute")): ', mezr.offset(target, mezr.containingBlock(this.scrollspyEl, 'absolute')));
                    // // console.log('mezr.offset(target, mezr.containingBlock(target, "fixed")): ', mezr.offset(target, mezr.containingBlock(target, 'fixed')));
                    // console.log('target.offsetTop: ', target.offsetTop);
                    // // console.log('target.position(): ', target.position());
                    // console.log('window.getComputedStyle(target).top: ', window.getComputedStyle(target).top);
                    // console.log('window.getComputedStyle(target).position: ', window.getComputedStyle(target).position);
                    // // console.log('window.getComputedStyle(target).offset: ', (window.getComputedStyle(target) as any).offset);
                    // console.log('targetBCR.top - scrollspyBCR.top: ', targetBCR.top - scrollspyBCR.top);
                    // console.log('offsetMethod: ', offsetMethod);
                    // console.log('offsetBase: ', offsetBase);
                    console.log('returnVal: ', returnVal);
                    return returnVal;
                }
            }
            return null;
        })
            .filter(function (item) { return item; })
            .sort(function (a, b) { return a[0] - b[0]; })
            .forEach(function (item) {
            _this.offsets.push(item[0]);
            _this.targets.push(item[1]);
        });
    };
    BsScrollspy.prototype.getScrollTop = function () {
        // return this.scrollElement === window ? this.scrollElement.pageYOffset : this.scrollElement.scrollTop;
        if (this.scrollElement === window) {
            return this.scrollElement.pageYOffset;
        }
        return this.scrollElement.scrollTop;
    };
    BsScrollspy.prototype.getScrollHeight = function () {
        return this.scrollElement.scrollHeight || Math.max(document.body.scrollHeight, document.documentElement.scrollHeight);
    };
    BsScrollspy.prototype.getOffsetHeight = function () {
        // if (this.useBodyForScrollElement) {
        //   return window.innerHeight;
        // }
        // return this.scrollElement.getBoundingClientRect().height;
        return this.scrollElement === window ? window.innerHeight : this.scrollElement.getBoundingClientRect().height;
    };
    BsScrollspy.prototype.isActiveTarget = function (index, scrollTop) {
        if (this.activeTarget === this.targets[index]) {
            return false;
        }
        if (scrollTop < this.offsets[index]) {
            return false;
        }
        if (typeof this.offsets[index + 1] === 'undefined') {
            if (this.verboseLogging) {
                console.log("at end of list activating \"" + this.targets[index] + "\"");
            }
            return true;
        }
        if (scrollTop < this.offsets[index + 1]) {
            if (this.verboseLogging) {
                console.log("scrollTop of " + scrollTop + " in range of " + this.offsets[index] + " - " + this.offsets[index + 1] + " activating \"" + this.targets[index] + "\"");
            }
            return true;
        }
        return false;
    };
    BsScrollspy.prototype.process = function () {
        if (!this.config.target || _size(this.config.target) === 0) {
            return;
        }
        var scrollTop = this.getScrollTop() + this.config.offset;
        if (this.verboseLogging) {
            console.log('scrollTop + offset = ', scrollTop);
        }
        var scrollHeight = this.getScrollHeight();
        var maxScroll = this.config.offset + scrollHeight - this.getOffsetHeight();
        var container = document.querySelector(this.config.target);
        if (this.scrollHeight !== scrollHeight) {
            this.refresh();
        }
        if (scrollTop >= maxScroll) {
            var target = this.targets[this.targets.length - 1];
            if (this.activeTarget !== target) {
                this.activate(target);
            }
            return;
        }
        if (this.activeTarget && scrollTop < this.offsets[0] && this.offsets[0] > 0) {
            this.activeTarget = null;
            this.clear(container);
            return;
        }
        for (var i = _size(this.offsets) - 1; i >= 0; i -= 1) {
            if (this.isActiveTarget(i, scrollTop)) {
                this.activate(this.targets[i]);
            }
        }
    };
    BsScrollspy.prototype.clear = function (container) {
        var nodeArr = Array.prototype.slice.call(container.querySelectorAll(this.selector));
        for (var j = 0; j < nodeArr.length; j += 1) {
            if (hasClass(nodeArr[j], 'active')) {
                removeClass(nodeArr[j], 'active');
            }
        }
    };
    BsScrollspy.getLinkArr = function (target, container) {
        var activeTargetEl = document.querySelector(target);
        if (!activeTargetEl) {
            return [];
        }
        var matchingLinks = [];
        var possibleLinkArr = Array.prototype.slice.call(container.querySelectorAll('.nav-link, .list-group-item, .dropdown-item'));
        for (var j = 0; j < possibleLinkArr.length; j += 1) {
            var targetSelector = getTargetSelector(possibleLinkArr[j]);
            if (targetSelector && elementMatches(activeTargetEl, targetSelector)) {
                matchingLinks.push(possibleLinkArr[j]);
            }
        }
        return matchingLinks;
    };
    BsScrollspy.getParentArrBySelector = function (el, selector) {
        var returnArr = [];
        var closestMatching;
        var currentEl = el.parentElement || el.parentNode;
        do {
            closestMatching = closest(currentEl, selector);
            if (closestMatching) {
                returnArr.push(closestMatching);
                currentEl = closestMatching.parentElement || closestMatching.parentNode;
            }
        } while (closestMatching);
        return returnArr;
    };
    BsScrollspy.prototype.activate = function (target) {
        if (!this.config.target || _size(this.config.target) === 0) {
            return;
        }
        this.activeTarget = target;
        var container = document.querySelector(this.config.target);
        this.clear(container);
        var linkArr = BsScrollspy.getLinkArr(target, container);
        for (var j = 0; j < linkArr.length; j += 1) {
            if (hasClass(linkArr[j], 'dropdown-item')) {
                var closestDropdown = closest(linkArr[j], '.dropdown');
                if (closestDropdown && container.contains(closestDropdown)) {
                    var dropdownToggle = closestDropdown.querySelector('.dropdown-toggle');
                    if (dropdownToggle) {
                        addClass(dropdownToggle, 'active');
                    }
                }
                addClass(linkArr[j], 'active');
            }
            else {
                addClass(linkArr[j], 'active');
                var parentSelector = '.nav, .list-group';
                var currentEl = linkArr[j];
                do {
                    currentEl = currentEl.parentElement || currentEl.parentNode;
                    currentEl = closest(currentEl, parentSelector);
                    var prevEl = currentEl ? currentEl.previousElementSibling : null;
                    if (currentEl && prevEl) {
                        if (elementMatches(prevEl, '.nav-link, .list-group-item')) {
                            addClass(currentEl.previousElementSibling, 'active');
                        }
                        // Handle special case when .nav-link is inside .nav-item
                        if (elementMatches(prevEl, '.nav-item')) {
                            var listItemChildren = Array.prototype.slice.call(prevEl.children);
                            for (var x = 0; x < listItemChildren.length; x += 1) {
                                if (elementMatches(listItemChildren[x], '.nav-link')) {
                                    addClass(listItemChildren[x], 'active');
                                }
                            }
                        }
                    }
                } while (currentEl && container.contains(currentEl));
            }
        }
        customEvent(this.scrollElement, this.activateEventName, {}, target);
    };
    BsScrollspy.prototype.scrollspy = function (scrollspyOptions) {
        if (scrollspyOptions === void 0) { scrollspyOptions = {}; }
        if (_size(scrollspyOptions) === 0) {
            return this.scrollspyEl;
        }
        if (scrollspyOptions === 'refresh') {
            this.refresh();
            return true;
        }
        if (scrollspyOptions === 'getActiveTarget') {
            return this.activeTarget;
        }
        if (typeof scrollspyOptions === 'object') {
            this.config = this.getConfig(scrollspyOptions);
            this.refresh();
            this.process();
            return true;
        }
        if (typeof scrollspyOptions === 'string') {
            throw new Error("No method named \"" + scrollspyOptions + "\"");
        }
        return null;
    };
    BsScrollspy.prototype.render = function () {
        return (h("slot", null));
    };
    Object.defineProperty(BsScrollspy, "is", {
        get: function () { return "bs-scrollspy"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsScrollspy, "properties", {
        get: function () {
            return {
                "activateEventName": {
                    "type": String,
                    "attr": "activate-event-name"
                },
                "activeTarget": {
                    "state": true
                },
                "config": {
                    "state": true
                },
                "defaults": {
                    "type": "Any",
                    "attr": "defaults"
                },
                "offsets": {
                    "state": true
                },
                "scrollElement": {
                    "state": true
                },
                "scrollHeight": {
                    "state": true
                },
                "scrollspy": {
                    "method": true
                },
                "scrollspyEl": {
                    "elementRef": true
                },
                "selector": {
                    "state": true
                },
                "targets": {
                    "state": true
                },
                "useBodyForScrollElement": {
                    "type": Boolean,
                    "attr": "use-body-for-scroll-element"
                },
                "verboseLogging": {
                    "type": Boolean,
                    "attr": "verbose-logging"
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsScrollspy, "style", {
        get: function () { return "bs-scrollspy {\n  display: block;\n}"; },
        enumerable: true,
        configurable: true
    });
    return BsScrollspy;
}());
export { BsScrollspy };
