/*! Built with http://stenciljs.com */
import { h } from './bscomponents.core.js';
import { p as _size, t as hasClass, s as addClass, q as customEvent, r as removeClass } from './chunk-89dac778.js';
import { a as getTransitionDurationFromElement } from './chunk-8f426baa.js';
import { a as reflow } from './chunk-89796593.js';
import { a as closest, b as elementMatches } from './chunk-5028a50e.js';
import { a as getTargetSelector } from './chunk-e1923c80.js';
import './chunk-64cf3b78.js';
import './chunk-5260d582.js';
var BsTab = /** @class */ (function () {
    function BsTab() {
        this.showEventName = 'show.bs.tab';
        this.shownEventName = 'shown.bs.tab';
        this.hideEventName = 'hide.bs.tab';
        this.hiddenEventName = 'hidden.bs.tab';
        this.ignoreDataToggles = false;
        this.dispatchEventsOnTab = false;
        this.showTab = false;
    }
    BsTab.prototype.componentWillLoad = function () {
        var _this = this;
        if (!this.showTab) {
            return;
        }
        var hasTransition = hasClass(this.tabEl, 'fade');
        if (!hasTransition) {
            this.show();
            return;
        }
        removeClass(this.tabEl, 'fade');
        this.tabEl.addEventListener(this.shownEventName, function () {
            addClass(_this.tabEl, 'fade');
        }, { once: true });
        this.show();
    };
    BsTab.prototype.show = function (triggeringButton) {
        var _this = this;
        if (triggeringButton === void 0) { triggeringButton = null; }
        if (hasClass(this.tabEl, 'disabled')) {
            return;
        }
        var toggler;
        if (triggeringButton) {
            toggler = triggeringButton;
        }
        else if (!this.ignoreDataToggles && !triggeringButton) {
            toggler = this.searchForTriggeringButton();
        }
        else {
            toggler = null;
        }
        if (!this.ignoreDataToggles && toggler && hasClass(toggler, 'disabled')) {
            return;
        }
        if (this.tabEl.parentNode && this.tabEl.parentNode.nodeType === Node.ELEMENT_NODE && hasClass(this.tabEl, 'active')) {
            return;
        }
        var previousActiveTabs = BsTab.getActiveElements(this.tabEl, this.tabEl.parentNode);
        var previousActiveButtons = this.getActiveButtons(toggler);
        var eventArr = [];
        var relatedTarget;
        if (this.dispatchEventsOnTab || this.ignoreDataToggles || !toggler) {
            for (var j = 0, len = previousActiveTabs.length; j < len; j += 1) {
                relatedTarget = previousActiveTabs[j];
                eventArr.push(customEvent(previousActiveTabs[j], this.hideEventName, {}, this.tabEl));
            }
            eventArr.push(customEvent(this.tabEl, this.showEventName, {}, relatedTarget));
        }
        else {
            for (var j = 0, len = previousActiveButtons.length; j < len; j += 1) {
                relatedTarget = previousActiveButtons[j];
                eventArr.push(customEvent(previousActiveButtons[j], this.hideEventName, {}, toggler));
            }
            eventArr.push(customEvent(toggler, this.showEventName, {}, relatedTarget));
        }
        if (eventArr.some(function (showOrHideEvent) { return showOrHideEvent.defaultPrevented; })) {
            return;
        }
        if (!this.ignoreDataToggles && toggler) {
            BsTab.activate(toggler, previousActiveButtons);
        }
        BsTab.activate(this.tabEl, previousActiveTabs, function () {
            if (_this.dispatchEventsOnTab || _this.ignoreDataToggles || !toggler) {
                for (var x = 0, len = previousActiveTabs.length; x < len; x += 1) {
                    customEvent(previousActiveTabs[x], _this.hiddenEventName, {}, _this.tabEl);
                }
                customEvent(_this.tabEl, _this.shownEventName, {}, relatedTarget);
            }
            else {
                for (var x = 0, len = previousActiveButtons.length; x < len; x += 1) {
                    customEvent(previousActiveButtons[x], _this.hiddenEventName, {}, toggler);
                }
                customEvent(toggler, _this.shownEventName, {}, relatedTarget);
            }
        });
    };
    BsTab.prototype.searchForTriggeringButton = function () {
        var matchingTriggers = [];
        var tabTriggerArr = Array.prototype.slice.call(document.querySelectorAll('[data-toggle="tab"], [data-toggle="pill"], [data-toggle="list"]'));
        for (var j = 0, len = tabTriggerArr.length; j < len; j += 1) {
            var targetSelector = getTargetSelector(tabTriggerArr[j]);
            if (targetSelector && elementMatches(this.tabEl, targetSelector)) {
                matchingTriggers.push(tabTriggerArr[j]);
            }
        }
        // const pillTriggerArr = Array.prototype.slice.call(document.querySelectorAll('[data-toggle="pill"]'));
        // for (let j = 0, len = pillTriggerArr.length; j < len; j += 1) {
        //   const targetSelector = getTargetSelector(pillTriggerArr[j]);
        //   if (targetSelector && elementMatches(this.tabEl, targetSelector)) {
        //     matchingTriggers.push(pillTriggerArr[j]);
        //   }
        // }
        // const listTriggerArr = Array.prototype.slice.call(document.querySelectorAll('[data-toggle="list"]'));
        // for (let j = 0, len = listTriggerArr.length; j < len; j += 1) {
        //   const targetSelector = getTargetSelector(listTriggerArr[j]);
        //   if (targetSelector && elementMatches(this.tabEl, targetSelector)) {
        //     matchingTriggers.push(listTriggerArr[j]);
        //   }
        // }
        if (matchingTriggers.length > 1) {
            console.warn('Unable to find tab toggle because more than one data toggle target matches this tab');
            return null;
        }
        if (matchingTriggers.length === 1) {
            return matchingTriggers[0];
        }
        return null;
    };
    BsTab.prototype.getActiveButtons = function (element) {
        if (this.ignoreDataToggles || !element) {
            return [];
        }
        var listElement = closest(element, '.nav, .list-group');
        if (listElement) {
            if (listElement.nodeName !== 'UL') {
                return Array.prototype.slice.call(listElement.querySelectorAll('.active'));
            }
            var activeButtonElArr = [];
            var listChildren = Array.prototype.slice.call(listElement.children);
            for (var j = 0, len = listChildren.length; j < len; j += 1) {
                if (listChildren[j].nodeName === 'LI') {
                    var activeInLi = Array.prototype.slice.call(listChildren[j].querySelectorAll('.active'));
                    for (var x = 0, len2 = activeInLi.length; x < len2; x += 1) {
                        activeButtonElArr.push(activeInLi[x]);
                    }
                }
            }
            return activeButtonElArr;
        }
        return [];
    };
    BsTab.getActiveElements = function (element, container) {
        if (container.nodeName === 'UL') {
            return Array.prototype.slice.call(element.parentNode.querySelectorAll('> li > .active'));
        }
        var activeChildren = [];
        var children = Array.prototype.slice.call(element.parentNode.children);
        for (var j = 0, len = children.length; j < len; j += 1) {
            if (hasClass(children[j], 'active')) {
                activeChildren.push(children[j]);
            }
        }
        return activeChildren;
    };
    BsTab.activate = function (element, activeElements, callback) {
        if (callback === void 0) { callback = function () { }; }
        var active = activeElements[0];
        var isTransitioning = active && hasClass(active, 'fade');
        var closeTabTransitionDuration = getTransitionDurationFromElement(active);
        // close other tabs
        for (var j = 0, len = activeElements.length; j < len; j += 1) {
            removeClass(activeElements[j], 'show');
            removeClass(activeElements[j], 'active');
            var dropdownChild = activeElements[j].parentNode.querySelectorAll('.dropdown-menu .active');
            for (var x = 0, len2 = dropdownChild.length; x < len2; x += 1) {
                removeClass(dropdownChild[x], 'active');
            }
            if (active.getAttribute('role') === 'tab') {
                active.setAttribute('aria-selected', false);
            }
        }
        // open this tab
        addClass(element, 'active');
        if (element.getAttribute('role') === 'tab') {
            element.setAttribute('aria-selected', true);
        }
        reflow(element);
        addClass(element, 'show');
        if (element.parentNode && hasClass(element.parentNode, 'dropdown-menu')) {
            var dropdownElement = closest(element, '.dropdown');
            if (dropdownElement) {
                var dropdownToggleList = Array.prototype.slice.call(dropdownElement.querySelectorAll('.dropdown-toggle'));
                for (var i = 0, len3 = dropdownToggleList.length; i < len3; i += 1) {
                    addClass(dropdownToggleList[i], 'active');
                }
            }
            element.setAttribute('aria-expanded', true);
        }
        if (isTransitioning) {
            setTimeout(function () {
                callback();
            }, closeTabTransitionDuration);
        }
        else {
            callback();
        }
    };
    BsTab.prototype.handleShowTabWatch = function (newValue /* , oldValue */) {
        if (newValue) {
            this.show();
        }
    };
    BsTab.prototype.tab = function (tabOptions, triggeringButton) {
        if (tabOptions === void 0) { tabOptions = {}; }
        if (triggeringButton === void 0) { triggeringButton = null; }
        if (_size(tabOptions) === 0) {
            return this.tabEl;
        }
        if (tabOptions === 'show') {
            this.show(triggeringButton);
            return true;
        }
        if (typeof tabOptions === 'string') {
            throw new Error("No method named \"" + tabOptions + "\"");
        }
        return null;
    };
    BsTab.prototype.render = function () {
        return (h("slot", null));
    };
    Object.defineProperty(BsTab, "is", {
        get: function () { return "bs-tab"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsTab, "properties", {
        get: function () {
            return {
                "dispatchEventsOnTab": {
                    "type": Boolean,
                    "attr": "dispatch-events-on-tab",
                    "mutable": true
                },
                "hiddenEventName": {
                    "type": String,
                    "attr": "hidden-event-name"
                },
                "hideEventName": {
                    "type": String,
                    "attr": "hide-event-name"
                },
                "ignoreDataToggles": {
                    "type": Boolean,
                    "attr": "ignore-data-toggles",
                    "mutable": true
                },
                "showEventName": {
                    "type": String,
                    "attr": "show-event-name"
                },
                "shownEventName": {
                    "type": String,
                    "attr": "shown-event-name"
                },
                "showTab": {
                    "type": Boolean,
                    "attr": "show-tab",
                    "mutable": true,
                    "watchCallbacks": ["handleShowTabWatch"]
                },
                "tab": {
                    "method": true
                },
                "tabEl": {
                    "elementRef": true
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsTab, "style", {
        get: function () { return "bs-tab {\n  display: block;\n}"; },
        enumerable: true,
        configurable: true
    });
    return BsTab;
}());
export { BsTab };
